// CLIPBOARD 1.0
// Copyleft 2005 by Volker Birk
// This program is under the GNU General Public License 2.0, see GPL.TXT

#include <io.h>
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

#define COPY 1
#define PASTE 2

#define MEMORY_PAGE_SIZE 1024

int nCode = 0;

int main(int c, char **argv)
{
	int i;
	HANDLE hMem;
	size_t nRead;
	char *pBuffer;
	int nCount = 0;

	for (i=1; i<c; i++)
		if (argv[i][0] == '/')
			switch (argv[i][1]) {
				case 'c':
				case 'C':
					nCode = COPY;
					break;
				case 'p':
				case 'P':
					nCode = PASTE;
					break;
				case 'h':
				case 'H':
				case '?':
					puts("CLIPBOARD /Copy | /Paste\n\nCopy from stdin or Paste to stdout");
					puts("\nCopyleft 2005 by Volker Birk. This program is under the GNU Public License 2.0."); 
					exit(0);
				case 'v':
				case 'V':
					puts("CLIPBOARD Version 1.0");
					exit(0);
				default:
					fprintf(stderr, "Illegal switch: /%c\n", argv[i][1]);
					exit(1);
			}
		else {
			fprintf(stderr, "Illegal parameter\n");
			exit(1);
		}

	if (nCode == 0) {
		fprintf(stderr, "Missing switch: /Copy or /Paste have to be applied\n");
		exit(1);
	}

	switch (nCode) {
		case COPY:
			if (OpenClipboard(NULL) == 0) {
				fprintf(stderr, "Cannot open clipboard\n");
				exit(2);
			}

			if (EmptyClipboard() == 0) {
				fprintf(stderr, "Cannot empty clipboard\n");
				exit(3);
			}

			if ((hMem = GlobalAlloc(GHND, MEMORY_PAGE_SIZE)) == NULL) {
				fprintf(stderr, "Cannot allocate memory for buffering data\n");
				exit(4);
			}

			do {
				BOOL bOverflow = FALSE;

				pBuffer = GlobalLock(hMem);

				nRead = 0;
				if (bOverflow)
					pBuffer[MEMORY_PAGE_SIZE * nCount + nRead++] = 10; // Windows nonsense

				do {
					char c = getchar();
					if (!feof(stdin)) {
						if (c == '\n')
							pBuffer[MEMORY_PAGE_SIZE * nCount + nRead++] = 13; // Windows nonsense
						if (nRead < MEMORY_PAGE_SIZE)
							pBuffer[MEMORY_PAGE_SIZE * nCount + nRead++] = c;
						else
							bOverflow = TRUE; // Windows nonsense
					}
				} while (!feof(stdin) && nRead < MEMORY_PAGE_SIZE);

				GlobalUnlock(hMem);

				if (nRead == MEMORY_PAGE_SIZE)
					if (GlobalReAlloc(hMem, MEMORY_PAGE_SIZE * (++nCount + 1), GMEM_ZEROINIT) == 0) {
						fprintf(stderr, "Cannot allocate memory for buffering data\n");
						CloseClipboard();
						exit(4);
					}
			} while (!feof(stdin));

			if (SetClipboardData(CF_TEXT, hMem) == NULL) {
				fprintf(stderr, "Cannot copy text to clipboard\n");
				CloseClipboard();
				exit(5);
			}

			CloseClipboard();
			break;

		case PASTE:
			if (OpenClipboard(NULL) == 0) {
				fprintf(stderr, "Cannot open clipboard\n");
				exit(2);
			}

			if ((hMem = GetClipboardData(CF_TEXT)) == NULL) {
				fprintf(stderr, "Cannot retrieve clipboard text\n");
				exit(5);
			}

			pBuffer = GlobalLock(hMem);
			for (i=0; pBuffer[i] != 0; i++) {
				if (pBuffer[i] == 13 && pBuffer[i+1] == 10) { // Windows nonsense
					putchar('\n');
					i++;
				} else
					putchar(pBuffer[i]);
			}
			GlobalUnlock(hMem);

			CloseClipboard();
			break;
	}

	return 0;
}
